-- board.e

include defs.e
include data.e

-- /* init() sets the board to the initial game state */

global
procedure init()

    Color=init_Color
    piece=init_piece       
    side=LIGHT 
    xside=DARK 
    castle=15
    ep=-1 
    fifty=0 
    ply=1
    gen_begin[1]=1
    gameover = 0
    counter = 0
    move_list = {}
    
end procedure


--/* attack returns TRUE if square sq is being attacked by side
--   s and FALSE otherwise. */


function attack(integer sq,integer s)

    integer n
    
    for i=1 to 64 do
        if Color[i] = s then    
            if piece[i] = PAWN then
                if s = DARK then
                    if FILE_[i] !=1 and i+7=sq then
                        return TRUE
                    elsif FILE_[i] !=8 and i+9=sq then
                        return TRUE
                    end if       
                else
                    if FILE_[i] != 1 and i-9 = sq then
                        return TRUE 
                    elsif FILE_[i] != 8 and i-7=sq then
                        return TRUE
                    end if
                end if
            else -- it is a piece
                for j=1 to length(offset[piece[i]+1]) do
                    n=i
                    while 1 do
                        n=mailbox[mailbox64[n]+offset[piece[i]+1][j]] 
                        if n=-1 then
                            exit 
                        elsif n=sq then
                            return TRUE 
                        elsif Color[n] != EMPTY then
                            exit                     
                        elsif  not slide[piece[i]+1] then
                            exit 
                        end if
                    end while
                end for
            end if
        end if
    end for

    return FALSE

end function


--/* in_check() returns TRUE if side s is in check and FALSE
--   otherwise. It just scans the board to find side s's king
--   and calls attack() to see if it's being attacked. */


global
function in_check(integer s)
    for i=1 to 64 do                            
        if Color[i] = s and piece[i] = KING then
            return attack(i,xor_bits(s,1))
        end if
    end for

    return 0

end function


--/* gen_promote() is just like gen_push(), only it puts 4 moves
--   on the move stack, one for each possible promotion piece */

procedure gen_promote(integer from,integer too,integer bits)

    for i=KNIGHT to QUEEN do
        gen_dat[gen_end[ply]][MOVE_][FROM]=from 
        gen_dat[gen_end[ply]][MOVE_][TO]=too
        gen_dat[gen_end[ply]][MOVE_][PROMOTE]=i
        gen_dat[gen_end[ply]][MOVE_][BITS]=or_bits(bits,32)
        gen_dat[gen_end[ply]][SCORE] = 1000000+(i*10) 
        gen_end[ply] += 1 
    end for

end procedure



--/* gen_push() puts a move on the move stack, unless it's a
--   pawn promotion that needs to be handled by gen_promote().
--   It also assigns a score to the move for alpha-beta move
--   ordering. If the move is a capture, it uses MVV/LVA
--   (Most Valuable Victim/Least Valuable Attacker). Otherwise,
--   it uses the move's history heuristic value. Note that
--   1,000,000 is added to a capture move's score, so it
--   always gets ordered above a "normal" move. */

procedure gen_push(integer from,integer too,integer bits)

    if and_bits(bits,16) then
        if side=LIGHT then
            if too <=8 then
                gen_promote(from,too,bits) 
                return 
            end if
        else 
            if too>=57  then
                gen_promote(from,too,bits) 
                return 
            end if
        end if
    end if 

    gen_dat[gen_end[ply]][MOVE_][FROM]=from 
    gen_dat[gen_end[ply]][MOVE_][TO]=too
    gen_dat[gen_end[ply]][MOVE_][PROMOTE]=0
    gen_dat[gen_end[ply]][MOVE_][BITS]=bits

    if and_bits(bits,1) then
        gen_dat[gen_end[ply]][SCORE]=1000000+(piece[too]*10)-piece[from]
    else
        gen_dat[gen_end[ply]][SCORE]=history[from][too] 
    end if

    gen_end[ply] += 1 

end procedure

--/* gen() generates pseudo-legal moves for the current position.
--   It scans the board to find friendly pieces and then determines
--   what squares they attack. When it finds a piece/square
--   combination, it calls gen_push to put the move on the "move
--   stack." */

procedure gen_dark()

    integer n

--  /* so far, we have no moves for the current ply */

    gen_end[ply]=gen_begin[ply]

    for i=1  to 64 do
        if Color[i]=DARK then
            if piece[i]=PAWN then
                if FILE_[i] !=1 and Color[i+7]=LIGHT then
                    gen_push(i,i+7,17) 
                end if                    
                if FILE_[i] !=8 and Color[i+9]=LIGHT then
                    gen_push(i,i+9,17) 
                end if
                if Color[i+8]=EMPTY then
                    gen_push(i,i+8,16) 
                    if i<=16 and Color[i+16]=EMPTY then
                        gen_push(i,i+16,24) 
                    end if
                end if
            else -- it is a piece
                for j=1 to length(offset[piece[i]+1]) do
                    n = i
                    while 1 do
                        n=mailbox[mailbox64[n]+offset[piece[i]+1][j]] 
                        if n=-1 then
                            exit
                        end if
                        if Color[n]!=EMPTY then
                            if Color[n]=xside then
                                gen_push(i,n,1) 
                            end if
                            exit
                        end if
                        gen_push(i,n,0) 
                        if  not slide[piece[i]+1] then
                            exit
                        end if
                    end while
                end for
            end if
        end if
    end for

    --  /* generate castle moves */

    if and_bits(castle,4) then
        gen_push(5,7,2) 
    end if
    if and_bits(castle,8) then
        gen_push(5,3,2)          
    end if

    --  /* generate en passant moves */

    if ep != -1 then
        if FILE_[ep] !=1 and Color[ep-9]=DARK and 
           piece[ep-9]=PAWN then
            gen_push(ep-9,ep,21) 
        end if
        if FILE_[ep] !=8 and Color[ep-7]=DARK and 
           piece[ep-7]=PAWN then
               gen_push(ep-7,ep,21)                
        end if
    end if
    
    -- /* the next ply's moves need to start where the current
    --    ply's end */

    gen_begin[ply+1]=gen_end[ply] 

end procedure


procedure gen_light()

    integer n

--  /* so far, we have no moves for the current ply */

    gen_end[ply]=gen_begin[ply]

    for i=1  to 64 do
        if Color[i]=LIGHT then
            if piece[i]=PAWN then
                if FILE_[i] !=1 and Color[i-9]=DARK then
                    gen_push(i,i-9,17)
                end if 
                if FILE_[i] !=8 and Color[i-7]=DARK then
                    gen_push(i,i-7,17) 
                end if
                if Color[i-8]=EMPTY then
                    gen_push(i,i-8,16) 
                    if i>=49 and Color[i-16]=EMPTY then
                        gen_push(i,i-16,24) 
                    end if
                end if
            else -- it is a piece
                for j=1 to length(offset[piece[i]+1]) do
                    n = i
                    while 1 do
                        n=mailbox[mailbox64[n]+offset[piece[i]+1][j]] 
                        if n=-1 then
                            exit
                        end if
                        if Color[n]!=EMPTY then
                            if Color[n]=xside then
                                gen_push(i,n,1) 
                            end if
                            exit
                        end if
                        gen_push(i,n,0) 
                        if  not slide[piece[i]+1] then
                            exit
                        end if
                    end while
                end for
            end if
        end if
    end for

    --  /* generate castle moves */

    if and_bits(castle,1) then
        gen_push(61,63,2) 
    end if
    if and_bits(castle,2) then
        gen_push(61,59,2)
    end if 

    --  /* generate en passant moves */

    if ep != -1 then
        if FILE_[ep] !=1 and Color[ep+7]=LIGHT 
           and piece[ep+7]=PAWN  then
            gen_push(ep+7,ep,21)     
        end if
        if FILE_[ep] !=8 and Color[ep+9]=LIGHT 
           and piece[ep+9]=PAWN then
            gen_push(ep+9,ep,21) 
        end if
    end if
    
    -- /* the next ply's moves need to start where the current
    --    ply's end */

    gen_begin[ply+1]=gen_end[ply] 

end procedure

global
procedure gen()
    if side = LIGHT then
        gen_light()
    else    
        gen_dark()
    end if
end procedure


--/* gen_caps() is basically a copy of gen() that's modified to
--   only generate capture and promote moves. It's used by the
--   quiescence search. */

global
procedure gen_caps_light()

    integer n 

    gen_end[ply]=gen_begin[ply] 

    for i=1 to 64 do
        if Color[i]=LIGHT then
            if piece[i]=PAWN then
                    if FILE_[i] !=1 and Color[i-9]=DARK then
                        gen_push(i,i-9,17)
                    end if 
                    if FILE_[i] !=8 and Color[i-7]=DARK then
                        gen_push(i,i-7,17) 
                    end if
                    if i<=16 and Color[i-8]=EMPTY then
                        gen_push(i,i-8,16) 
                    end if
            else  -- it is a piece
                for j=1 to length(offset[piece[i]+1]) do
                    n=i
                    while 1 do
                        n=mailbox[mailbox64[n]+offset[piece[i]+1][j]] 
                        if n=-1 then
                            exit 
                        end if
                        if Color[n]!=EMPTY then
                            if Color[n]=xside then
                                gen_push(i,n,1) 
                            end if
                            exit    
                        end if
                        if not slide[piece[i]+1] then
                            exit
                        end if
                    end while
                end for
            end if
        end if
    end for

    if ep!=-1 then
            if FILE_[ep] !=1 and Color[ep+7]=LIGHT and 
               piece[ep+7]=PAWN then
                gen_push(ep+7,ep,21) 
            end if
            if FILE_[ep] !=8 and Color[ep+9]=LIGHT and 
               piece[ep+9]=PAWN then
                gen_push(ep+9,ep,21) 
            end if
    end if         

    gen_begin[ply+1]=gen_end[ply]

end procedure       

procedure gen_caps_dark()

    integer n 

    gen_end[ply]=gen_begin[ply] 

    for i=1 to 64 do
        if Color[i]=DARK then
            if piece[i]=PAWN then
                    if FILE_[i] !=1 and Color[i+7]=LIGHT then
                        gen_push(i,i+7,17)                    
                    end if 
                    if FILE_[i] !=8 and Color[i+9]=LIGHT then
                        gen_push(i,i+9,17) 
                    end if
                    if i>=49 and Color[i+8]=EMPTY then
                        gen_push(i,i+8,16) 
                    end if   
            else  -- it is a piece
                for j=1 to length(offset[piece[i]+1]) do
                    n=i
                    while 1 do
                        n=mailbox[mailbox64[n]+offset[piece[i]+1][j]] 
                        if n=-1 then
                            exit 
                        end if
                        if Color[n]!=EMPTY then
                            if Color[n]=xside then
                                gen_push(i,n,1) 
                            end if
                            exit    
                        end if
                        if not slide[piece[i]+1] then
                            exit
                        end if
                    end while
                end for
            end if
        end if
    end for

    if ep!=-1 then
            if FILE_[ep] !=1 and Color[ep-9]=DARK and 
               piece[ep-9]=PAWN then
                gen_push(ep-9,ep,21) 
            end if
            if FILE_[ep] !=8 and Color[ep-7]=DARK and 
               piece[ep-7]=PAWN then
                gen_push(ep-7,ep,21) 
            end if
    end if         

    gen_begin[ply+1]=gen_end[ply]

end procedure

global
procedure gen_caps()
    if side = LIGHT then
        gen_caps_light()
    else    
        gen_caps_dark()
    end if
end procedure


--/* takeback() is very similar to makemove(), only backwards :)  */

global
procedure takeback()

    sequence m     
    integer from,too 

    side = xor_bits(side,1)
    xside = xor_bits(xside,1)

    ply -= 1             

    m=hist_dat[ply][MOVE_]
    castle=hist_dat[ply][CAS]
    ep=hist_dat[ply][EP]
    fifty=hist_dat[ply][FIFTY] 
    Color[m[FROM]]=side 
    
    if and_bits(m[BITS],32) then
        piece[m[FROM]]=PAWN 
    else
        piece[m[FROM]]=piece[m[TO]] 
    end if
    
    if hist_dat[ply][CAP]=EMPTY then
        Color[m[TO]]=EMPTY 
        piece[m[TO]]=EMPTY 
    else 
        Color[m[TO]]=xside 
        piece[m[TO]]=hist_dat[ply][CAP]
    end if
    
    if and_bits(m[BITS],2) then
        if m[TO] = 63 then
            from=62  too=64  
        elsif m[TO] = 59 then
            from=60  too=57 
        elsif m[TO] = 7 then
            from=6  too=8 
        elsif m[TO] = 3 then
            from=4  too=1 
        end if          
        Color[too]=side 
        piece[too]=ROOK 
        Color[from]=EMPTY 
        piece[from]=EMPTY 
    end if
    
    if and_bits(m[BITS],4) then
        if side=LIGHT then
            Color[m[TO]+8]=xside 
            piece[m[TO]+8]=PAWN 
        else 
            Color[m[TO]-8]=xside 
            piece[m[TO]-8]=PAWN 
        end if
    end if

end procedure


--/* makemove() makes a move. If the move is illegal, it
--   undoes whatever it did and returns FALSE. Otherwise, it
--   returns TRUE. */

global
function makemove(sequence m)

    -- /* test to see if a castle move is legal and move the rook
    --    (the king is moved with the usual move code later) */
    if and_bits(m[BITS],2) then
        if in_check(side) then
            return FALSE
        end if
        if m[TO] = 63 then
            if Color[62]!=EMPTY or Color[63]!=EMPTY or 
               attack(62,xside) or attack(63,xside) then
                return FALSE
            end if
            Color[62]=LIGHT 
            piece[62]=ROOK 
            Color[64]=EMPTY 
            piece[64]=EMPTY 
        elsif m[TO] = 59 then
            if Color[58]!=EMPTY or Color[59]!=EMPTY or Color[60]!=EMPTY or 
               attack(59,xside) or attack(60,xside) then
                return FALSE  
            end if
            Color[60]=LIGHT 
            piece[60]=ROOK 
            Color[57]=EMPTY 
            piece[57]=EMPTY 
        elsif m[TO] = 7 then
            if Color[6]!=EMPTY or Color[7]!=EMPTY or 
               attack(6,xside) or attack(7,xside) then
                return FALSE
            end if
            Color[6]=DARK 
            piece[6]=ROOK 
            Color[8]=EMPTY 
            piece[8]=EMPTY 
        elsif m[TO] = 3 then
            if Color[2]!=EMPTY or Color[3]!=EMPTY or Color[4]!=EMPTY or 
               attack(3,xside) or attack(4,xside) then
                return FALSE 
            end if
            Color[4]=DARK 
            piece[4]=ROOK 
            Color[1]=EMPTY 
            piece[1]=EMPTY 
        end if
    end if

    --  /* back up information so we can take the move back later. */

    hist_dat[ply][MOVE_]=m 
    hist_dat[ply][CAP]=piece[m[TO]] 
    hist_dat[ply][CAS]=castle 
    hist_dat[ply][EP]=ep 
    hist_dat[ply][FIFTY]=fifty 
    ply += 1 

    --/* update the castle, en passant, and
    --   fifty-move-draw variables */

    castle = and_bits(castle,
             and_bits(castle_mask[m[FROM]],castle_mask[m[TO]])) 

    if and_bits(m[BITS],8) then
        if side=LIGHT then
            ep=m[TO]+8 
        else
            ep=m[TO]-8 
        end if
    else
        ep=-1 
    end if

    if and_bits(m[BITS],17) then
        fifty=0 
    else
        fifty += 1 
    end if

    --/* move the piece */

    Color[m[TO]]=side 

    if and_bits(m[BITS],32) then 
        piece[m[TO]]=m[PROMOTE]
    else
        piece[m[TO]]=piece[m[FROM]] 
    end if
    
    Color[m[FROM]]=EMPTY 
    piece[m[FROM]]=EMPTY 

    --/* erase the pawn if this is an en passant move */

    if and_bits(m[BITS],4) then
        if side=LIGHT then
            Color[m[TO]+8]=EMPTY 
            piece[m[TO]+8]=EMPTY 
        else 
            Color[m[TO]-8]=EMPTY 
            piece[m[TO]-8]=EMPTY 
        end if
    end if

    --/* switch sides and test for legality (if we can capture
    --     the other guy's king, it's an illegal position and
    --       we need to take the move back) */

    side = xor_bits(side,1) 
    xside = xor_bits(xside,1) 

    if in_check(xside) then
        takeback() 
        return FALSE
    end if      

    return TRUE

end function

