-- data.e

include defs.e

--/* the board representation */
--/* PAWN, KNIGHT, BISHOP, ROOK, QUEEN, KING, or EMPTY */

global sequence Color,piece

--/* the side to move */

global integer side

--/* the side not to move */

global integer xside

--/* a bitfield with the castle permissions. if 1 is set,
--   white can still castle kingside. 2 is white queenside.
--   4 is black kingside. 8 is black queenside. */

global integer castle

--/* the en passant square. if white moves e2e4, the en passant
--   square is set to e3, because that's where a pawn would move
--   in an en passant capture */

global integer ep

--/* the number of moves since a capture or pawn move, used
--   to handle the fifty-move-draw rule */

global integer fifty

--/* the half-move that we're on */

global integer ply

--/* this is the move stack. gen_dat is basically a list of move lists,
--   all stored back to back. gen_begin[x] is where the first move of the
--   ply x move list is (in gen_dat). gen_end is right after the last move. */

global sequence gen_dat,gen_begin,gen_end
gen_dat   = repeat(GEN_REC,MOVE_STACK)
gen_begin = repeat(0,HIST_STACK)
gen_end   = repeat(0,HIST_STACK)


global sequence history
history = repeat(repeat(0,64),64)

--/* we need an array of hist_rec's so we can take back the
--   moves we make */

global sequence hist_dat
hist_dat = repeat(HIST_REC,HIST_STACK)

--/* the number of nodes we've searched */

global integer nodes

--/* a triangular PV array */

global sequence pv
--pv = repeat(repeat(MOVE_REC,HIST_STACK),HIST_STACK)
pv = repeat(repeat(MOVE_REC,100),100)

global sequence pv_length
--pv_length = repeat(0,HIST_STACK)
pv_length = repeat(0,100)

global integer follow_pv


--/* Now we have the mailbox array, so called because it looks like a
--   mailbox, at least according to Bob Hyatt. This is useful when we
--   need to figure out what pieces can go where. Let's say we have a
--   rook on square a4 (32) and we want to know if it can move one
--   square to the left. We subtract 1, and we get 31 (h5). The rook
--   obviously can't move to h5, but we don't know that without doing
--   a lot of annoying work. Sooooo, what we do is figure out a4's
--   mailbox number, which is 61. Then we subtract 1 from 61 (60) and
--   see what mailbox[60] is. In this case, it's -1, so it's out of
--   bounds and we can forget it. You can see how mailbox[] is used
--   in attack() in board.c. */
--

global constant mailbox = {
         -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
         -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
         -1,  1,  2,  3,  4,  5,  6,  7,  8, -1,
         -1,  9, 10, 11, 12, 13, 14, 15, 16, -1,
         -1, 17, 18, 19, 20, 21, 22, 23, 24, -1,
         -1, 25, 26, 27, 28, 29, 30, 31, 32, -1,
         -1, 33, 34, 35, 36, 37, 38, 39, 40, -1,
         -1, 41, 42, 43, 44, 45, 46, 47, 48, -1,
         -1, 49, 50, 51, 52, 53, 54, 55, 56, -1,
         -1, 57, 58, 59, 60, 61, 62, 63, 64, -1,
         -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
         -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
}

global constant mailbox64 = {
        22, 23, 24, 25, 26, 27, 28, 29,
        32, 33, 34, 35, 36, 37, 38, 39,
        42, 43, 44, 45, 46, 47, 48, 49,
        52, 53, 54, 55, 56, 57, 58, 59,
        62, 63, 64, 65, 66, 67, 68, 69,
        72, 73, 74, 75, 76, 77, 78, 79,
        82, 83, 84, 85, 86, 87, 88, 89,
        92, 93, 94, 95, 96, 97, 98, 99
}


--/* slide, offsets, and offset are basically the vectors that
--   pieces can move in. If slide for the piece is FALSE, it can
--   only move one square in any one direction. offsets is the
--   number of directions it can move in, and offset is an array
--   of the actual directions. */

global constant slide = {
        FALSE, FALSE, TRUE, TRUE, TRUE, FALSE
}

global constant offset= {
        {},
        {-21, -19, -12, -8, 8, 12, 19, 21},
        {-11, -9, 9, 11},
        {-10, -1, 1, 10},
        {-11, -10, -9, -1, 1, 9, 10, 11},
        {-11, -10, -9, -1, 1, 9, 10, 11},
        {}
}


--/* This is the castle_mask array. We can use it to determine
--   the castling permissions after a move. What we do is
--   logical-AND the castle bits with the castle_mask bits for
--   both of the move's squares. Let's say castle is 1, meaning
--   that white can still castle kingside. Now we play a move
--   where the rook on h1 gets captured. We AND castle with
--   castle_mask[63], so we have 1&14, and castle becomes 0 and
--   white can't castle kingside anymore. */

global constant castle_mask = {
         7, 15, 15, 15,  3, 15, 15, 11,
        15, 15, 15, 15, 15, 15, 15, 15,
        15, 15, 15, 15, 15, 15, 15, 15,
        15, 15, 15, 15, 15, 15, 15, 15,
        15, 15, 15, 15, 15, 15, 15, 15,
        15, 15, 15, 15, 15, 15, 15, 15,
        15, 15, 15, 15, 15, 15, 15, 15,
        13, 15, 15, 15, 12, 15, 15, 14
}


--/* the initial board state */

global constant init_Color = {
        1, 1, 1, 1, 1, 1, 1, 1,
        1, 1, 1, 1, 1, 1, 1, 1,
        6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0
}

global constant init_piece = {
        3, 1, 2, 4, 5, 2, 1, 3,
        0, 0, 0, 0, 0, 0, 0, 0,
        6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6,
        6, 6, 6, 6, 6, 6, 6, 6,
        0, 0, 0, 0, 0, 0, 0, 0,
        3, 1, 2, 4, 5, 2, 1, 3
}

global integer gameover,counter

global sequence move_list


