-- eval.e

include defs.e
include data.e 


constant DOUBLED_PAWN_PENALTY       = 10
constant ISOLATED_PAWN_PENALTY      = 20
constant BACKWARDS_PAWN_PENALTY     =  8
constant PASSED_PAWN_BONUS          = 20
constant ROOK_SEMI_OPEN_FILE_BONUS  = 10
constant ROOK_OPEN_FILE_BONUS       =  5
constant ROOK_ON_SEVENTH_BONUS      = 20

-- /* the values of the pieces */
--        100, 350, 375, 550, 1000, 0
--        100, 300, 325, 500, 950, 0

constant piece_value = {
        100, 300, 300, 500, 900, 0  -- original values
}

-- /* The "pcsq" arrays are piece/square tables. They're values
--    added to the material value of the piece based on the
--    location of the piece. */

constant pawn_pcsq= {
          0,   0,   0,   0,   0,   0,   0,   0,
          5,  10,  15,  20,  20,  15,  10,   5,
          4,   8,  12,  16,  16,  12,   8,   4,
          3,   6,   9,  12,  12,   9,   6,   3,
          2,   4,   6,   8,   8,   6,   4,   2,
          1,   2,   3, -10, -10,   3,   2,   1,
          0,   0,   0, -40, -40,   0,   0,   0,
          0,   0,   0,   0,   0,   0,   0,   0
} 

constant knight_pcsq= {
        -10, -10, -10, -10, -10, -10, -10, -10,
        -10,   0,   0,   0,   0,   0,   0, -10,
        -10,   0,   5,   5,   5,   5,   0, -10,
        -10,   0,   5,  10,  10,   5,   0, -10,
        -10,   0,   5,  10,  10,   5,   0, -10,
        -10,   0,   5,   5,   5,   5,   0, -10,
        -10,   0,   0,   0,   0,   0,   0, -10,
        -10, -30, -10, -10, -10, -10, -30, -10
} 

constant bishop_pcsq= {
        -10, -10, -10, -10, -10, -10, -10, -10,
        -10,   0,   0,   0,   0,   0,   0, -10,
        -10,   0,   5,   5,   5,   5,   0, -10,
        -10,   0,   5,  10,  10,   5,   0, -10,
        -10,   0,   5,  10,  10,   5,   0, -10,
        -10,   0,   5,   5,   5,   5,   0, -10,
        -10,   0,   0,   0,   0,   0,   0, -10,
        -10, -10, -20, -10, -10, -20, -10, -10
} 

constant king_pcsq= {
         -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
         -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
         -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
         -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
         -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
         -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
         -4,  -4,  -4,  -4,  -4,  -4,  -4,  -4,
          0,   4,   8,  -4,   0,  -4,   8,   4
} 

constant king_endgame_pcsq= {
          0,  10,  20,  30,  30,  20,  10,   0,
         10,  20,  30,  40,  40,  30,  20,  10,
         20,  30,  40,  50,  50,  40,  30,  20,
         30,  40,  50,  60,  60,  50,  40,  30,
         30,  40,  50,  60,  60,  50,  40,  30,
         20,  30,  40,  50,  50,  40,  30,  20,
         10,  20,  30,  40,  40,  30,  20,  10,
          0,  10,  20,  30,  30,  20,  10,   0
} 

-- /* The flip array is used to calculate the piece/square
--    values for DARK pieces. The piece/square value of a
--    LIGHT pawn is pawn_pcsq[sq] and the value of a DARK
--    pawn is pawn_pcsq[flip[sq]] */

constant flip= {
         57,  58,  59,  60,  61,  62,  63,  64,
         49,  50,  51,  52,  53,  54,  55,  56,
         41,  42,  43,  44,  45,  46,  47,  48,
         33,  34,  35,  36,  37,  38,  39,  40,
         25,  26,  27,  28,  29,  30,  31,  32,
         17,  18,  19,  20,  21,  22,  23,  24,
          9,  10,  11,  12,  13,  14,  15,  16,
          1,   2,   3,   4,   5,   6,   7,   8
} 

-- /* pawn_count[x][y] is the number of pawns of Color x on file y.
--    The array has two extra files (0 and 9) that will always be 0
--    so we can use the same pawn evaluation code on every pawn without
--    worrying about going out of bounds in the array. */

sequence pawn_count

-- /* pawn_rank[x][y] is the rank of the least advanced pawn of Color x
--    on file y. If there isn't a pawn on that file, we assume it
--    advanced off the board. This is a kludge that makes some of the
--    pawn evaluation code work. This array has two extra files for the
--    same reason pawn_count does. */

sequence pawn_rank

sequence piece_mat --  /* the value of a side's pieces */
piece_mat = {0,0}

sequence pawn_mat --  /* the value of a side's pawns */
pawn_mat = {0,0}


function eval_light_pawn(integer sq)

    integer r   --/* the value to return */
    integer f   --/* the pawn's file */

    f=FILE_[sq]+1 

    r = pawn_pcsq[sq] 

    --/* if there's a pawn behind this one, it's doubled */

    if pawn_rank[LIGHT_0][f]>RANK_[sq] then
        r-=DOUBLED_PAWN_PENALTY 
    end if

    --/* if there aren't any friendly pawns on either side of
    --   this one, it's isolated */

    if not pawn_count[LIGHT_0][f-1] and  not pawn_count[LIGHT_0][f+1] then
        r-=ISOLATED_PAWN_PENALTY 
    -- /* if it's not isolated, it might be backwards */
    elsif pawn_rank[LIGHT_0][f-1]<RANK_[sq] and pawn_rank[LIGHT_0][f+1]<RANK_[sq] then
        r-=BACKWARDS_PAWN_PENALTY 
    end if

    --/* add a bonus if the pawn is passed */

    if pawn_rank[DARK_1][f-1]>=RANK_[sq] and pawn_rank[DARK_1][f]>=RANK_[sq] and
       pawn_rank[DARK_1][f+1]>=RANK_[sq] then
        r+=(9-RANK_[sq])*PASSED_PAWN_BONUS 
    end if

    return r 

end function

function eval_dark_pawn(integer sq)

    integer r   --/* the value to return */
    integer f   --/* the pawn's file */

    f=FILE_[sq]+1 

    r = pawn_pcsq[flip[sq]] 

    --/* if there's a pawn behind this one, it's doubled */

    if pawn_rank[DARK_1][f]<RANK_[sq] then
        r-=DOUBLED_PAWN_PENALTY 
    end if

    -- /* if there aren't any friendly pawns on either side of
    --    this one, it's isolated */

    if not pawn_count[DARK_1][f-1] and not pawn_count[DARK_1][f+1] then
        r-=ISOLATED_PAWN_PENALTY 
    --/* if it's not isolated, it might be backwards */
    elsif pawn_rank[DARK_1][f-1]>RANK_[sq] and pawn_rank[DARK_1][f+1]>RANK_[sq] then
        r-=BACKWARDS_PAWN_PENALTY 
    end if

    --/* add a bonus if the pawn is passed */

    if pawn_rank[LIGHT_0][f-1]<=RANK_[sq] and  pawn_rank[LIGHT_0][f]<=RANK_[sq] and
       pawn_rank[LIGHT_0][f+1]<=RANK_[sq] then
        r+=9-RANK_[sq]*PASSED_PAWN_BONUS 
    end if

    return r

end function 


-- /* The following two functions are the king safety functions.
--    Here's the idea:
--    First, the piece/square value is added to r (r is the value
--      that's returned)
--    Second, the pawns in front of the king are evaluated. If the
--      king is on the queenside, the pawns on files a, b, and c
--          are evaluated. If the king is on the kingside, the pawns
--          on files f, g, and h are evaluated. The evaluation of each
--          file works like this: if there isn't a friendly pawn on the
--          file, it's considered open and a penalty is assessed. If
--          there is a friendly pawn on the file, and it hasn't been
--          pushed, nothing happens. If it's been pushed one square,
--          a penalty is assessed. If it's been pushed more than one
--          square, a penalty (usually bigger) is assessed. After this
--          is done for the three files, the enemy pawns are examed.
--          If they're getting too close to the king, it's a "pawn
--          storm" and penalties are assessed. If the king isn't on
--          the kingside or the queenside, i.e., if it's on one of the
--          two center files, then the three files closest to it are
--          examined. A penalty is assessed for each one of these files
--          that's empty.
--    Finally, the score is multiplied by a scalar of the opponent's
--      material. The rationale for this is that if there are problems
--          with your king safety and your opponent has a lot of pieces,
--          you're in trouble because it's easy for your opponent to
--          attack you. On the other hand, if you have problems with your
--          king safety and your opponent has no pieces, it doesn't really
--          matter, because he can't attack you. */
-- 

function eval_light_king(integer sq)

    integer r   --/* the value to return */

    r=king_pcsq[sq] 

    if FILE_[sq]<4 then
        --/* a file */
        if pawn_count[LIGHT_0][2] then
            if pawn_rank[LIGHT_0][2] = 7 then
                -- continue
            else 
                if pawn_rank[LIGHT_0][2] = 6 then  
                    r-=2 
                else
                    r-=4 
                end if
            end if
        elsif not pawn_count[DARK_1][2] then
            r-=8 
        end if
        --/* b file */
        if pawn_count[LIGHT_0][3] then
            if pawn_rank[LIGHT_0][3] = 7 then
                -- continue
            elsif pawn_rank[LIGHT_0][3] = 6 then
                r-=2 
            else
                r-=4 
            end if
        elsif not pawn_count[DARK_1][3] then
            r-=8 
        end if
        --/* c file */
        if pawn_count[LIGHT_0][4] then
            if pawn_rank[LIGHT_0][4] = 7 then
                -- continue
            elsif pawn_rank[LIGHT_0][4] = 6 then
                r-=1 
            else
                r-=3 
            end if
        elsif not pawn_count[DARK_1][4] then
            r-=5 
        end if
        --/* pawn storm */
        for i=2 to 4 do
            if not pawn_count[DARK_1][i] then
              --continue 
            else
                if pawn_rank[DARK_1][i]>3 then
                    if pawn_rank[DARK_1][i]>4 then
                        r-=2 
                    else
                        r-=1 
                    end if
                end if
            end if
        end for
    elsif FILE_[sq]>5 then
        --/* h file */
        if pawn_count[LIGHT_0][9] then
            if pawn_rank[LIGHT_0][9] = 7 then
                -- continue
            elsif pawn_rank[LIGHT_0][9] = 6 then
                r-=2 
            else
                r-=4 
            end if
        elsif not pawn_count[DARK_1][9] then
            r-=8 
            --/* g file */
            if pawn_count[LIGHT_0][8] then
                if pawn_rank[LIGHT_0][8] = 7 then
                    -- continue
                elsif(pawn_rank[LIGHT_0][8] = 6) then
                    r-=2 
                else
                    r-=4 
                end if
            elsif not pawn_count[DARK_1][8] then
                r-=8 
            end if
            --/* f file */
            if pawn_count[LIGHT_0][7] then
                if pawn_rank[LIGHT_0][7] = 7 then
                    -- continue
                elsif pawn_rank[LIGHT_0][7] = 6 then
                    r-=1 
                else
                    r-=3 
                end if
            elsif not pawn_count[DARK_1][7] then
                r-=5 
            end if
            --/* pawn storm */
            for i=7 to 9 do
                if not pawn_count[DARK_1][i] then
                    --continue 
                else                        
                    if pawn_rank[DARK_1][i]>3 then
                        if pawn_rank[DARK_1][i]>4 then
                            r-=2 
                        else
                            r-=1 
                        end if
                    end if
                end if
            end for
        end if
    else 
        for i=FILE_[sq] to FILE_[sq]+2 do
            if not pawn_count[LIGHT_0][i] and not pawn_count[DARK_1][i] then
                r-=2 
            end if
        end for
    end if
 
    r*=floor(piece_mat[DARK_1]/600)

    return r

end function

function eval_dark_king(integer sq)

    integer r   --/* the value to return */

    r=king_pcsq[flip[sq]] 

    if FILE_[sq]<4 then
        --/* a file */
        if pawn_count[DARK_1][2] then
            if pawn_rank[DARK_1][2] = 2 then
                -- continue
            elsif pawn_rank[DARK_1][2] = 3 then
                r-=2 
            else
                r-=4 
            end if
        elsif not pawn_count[LIGHT_0][3] then
            r-=8 
        end if
        --/* b file */
        if pawn_count[DARK_1][3] then
            if pawn_rank[DARK_1][3] = 2 then
                -- continue
            elsif pawn_rank[DARK_1][3] = 3 then
                r-=2 
            else
                r-=4 
            end if
        elsif not pawn_count[LIGHT_0][3] then
            r-=8 
        end if 
        --/* c file */
        if pawn_count[DARK_1][4] then
            if pawn_rank[DARK_1][4] = 2 then
                -- continue
            elsif pawn_rank[DARK_1][4] = 3 then
                r-=1 
            else
                r-=3 
            end if
        elsif not pawn_count[LIGHT_0][4] then
            r-=5 
        end if
        --/* pawn storm */
        for i=2 to 4 do
            if not pawn_count[LIGHT_0][i] then
                --continue 
            else
                if pawn_rank[LIGHT_0][i]<6 then
                    if pawn_rank[DARK_1][i]<5 then
                        r-=2 
                    else
                        r-=1 
                    end if
                end if
            end if
        end for
    elsif FILE_[sq]>5 then
        --/* h file */
        if pawn_count[DARK_1][9] then
            if pawn_rank[DARK_1][9] = 2 then
                -- continue
            elsif pawn_rank[DARK_1][9] = 3 then
                r-=2 
            else
                r-=4 
            end if
        elsif not pawn_count[LIGHT_0][9] then
            r-=8 
        end if
        --/* g file */
        if pawn_count[DARK_1][8] then
            if pawn_rank[DARK_1][8] = 2 then
                -- continue
            elsif pawn_rank[DARK_1][8] = 3 then
                r-=2 
            else
                r-=4 
            end if
        elsif not pawn_count[LIGHT_0][8] then
            r-=8 
        end if
        --/* f file */
        if pawn_count[DARK_1][7] then
            if pawn_rank[DARK_1][7] = 2 then
                -- continue
            elsif pawn_rank[DARK_1][7] = 3 then
                r-=1 
            else
                r-=3 
            end if
        elsif not pawn_count[LIGHT_0][7] then
            r-=5 
        end if
        --/* pawn storm */
        for i=7 to 9 do
            if not pawn_count[LIGHT_0][i] then
                --continue 
            else
                if pawn_rank[LIGHT_0][i]<6 then
                    if pawn_rank[LIGHT_0][i]<5 then
                        r-=2 
                    else
                        r-=1 
                    end if
                end if
            end if
        end for
    else 
        for i=FILE_[sq] to FILE_[sq]+2 do
            if not pawn_count[LIGHT_0][i] and not pawn_count[DARK_1][i] then
                r-=2 
            end if
        end for
    end if

    r*=floor(piece_mat[LIGHT_0]/600)

    return r         

end function


global 
function eval()

    integer f   --/* file */
    sequence score   --/* each side's score */

    score = {0,0}

    --/* this is the first pass: set up pawn_count, pawn_rank,
    --   piece_mat, and pawn_mat. */

    pawn_count  = repeat(repeat(0,10),2)
    pawn_rank  = repeat(repeat(0,10),2)

    pawn_rank[DARK_1]  += 9 

    piece_mat[LIGHT_0]=0 
    piece_mat[DARK_1]=0 
    pawn_mat[LIGHT_0]=0 
    pawn_mat[DARK_1]=0 

    for i=1 to 64 do
        if Color[i] = EMPTY then
            -- continue
        else
            if piece[i] = PAWN then
                pawn_mat[Color[i]+1] += piece_value[PAWN+1] 
                f=FILE_[i]+1   --/* add 1 because of the extra file in the array */
                if Color[i] = LIGHT then
                    pawn_count[LIGHT_0][f] += 1 
                    if pawn_rank[LIGHT_0][f]<RANK_[i] then
                        pawn_rank[LIGHT_0][f]=RANK_[i] 
                    end if
                else -- Color is DARK
                    pawn_count[DARK_1][f] += 1
                    if pawn_rank[DARK_1][f]>RANK_[i] then
                        pawn_rank[DARK_1][f]=RANK_[i] 
                    end if
                end if
            else -- piece not a pawn
                piece_mat[Color[i]+1]+=piece_value[piece[i]+1] 
            end if
        end if
    end for
--        /* this is the second pass: evaluate each piece */

    score[LIGHT_0]=piece_mat[LIGHT_0]+pawn_mat[LIGHT_0] 
    score[DARK_1]=piece_mat[DARK_1]+pawn_mat[DARK_1] 

    for i=1 to 64 do
        if Color[i] = EMPTY then
            -- continue
        else
            if Color[i] = LIGHT then
                if piece[i] = PAWN then
                    score[LIGHT_0]+=eval_light_pawn(i) 
                elsif piece[i] = KNIGHT then
                    score[LIGHT_0]+=knight_pcsq[i] 
                elsif piece[i] = BISHOP then
                    score[LIGHT_0]+=bishop_pcsq[i] 
                elsif piece[i] = ROOK then
                    if not pawn_count[LIGHT_0][FILE_[i]+1] then
                        score[LIGHT_0]+=ROOK_SEMI_OPEN_FILE_BONUS 
                        if not pawn_count[DARK_1][FILE_[i]+1] then
                            score[LIGHT_0]+=ROOK_OPEN_FILE_BONUS 
                        end if
                        if RANK_[i] = 2 then
                            score[LIGHT_0]+=ROOK_ON_SEVENTH_BONUS 
                        end if
                    end if
                elsif piece[i] = KING then
                    if piece_mat[DARK_1]<=1200 then
                        score[LIGHT_0]+=king_endgame_pcsq[i] 
                    else
                        score[LIGHT_0]+=eval_light_king(i) 
                    end if
                end if
            else -- DARK side
                if piece[i] = PAWN then
                    score[DARK_1]+=eval_dark_pawn(i) 
                elsif piece[i] = KNIGHT then
                    score[DARK_1]+=knight_pcsq[flip[i]] 
                elsif piece[i] = BISHOP then
                    score[DARK_1]+=bishop_pcsq[flip[i]] 
                elsif piece[i] = ROOK then
                    if not pawn_count[DARK_1][FILE_[i]+1] then
                        score[DARK_1]+=ROOK_SEMI_OPEN_FILE_BONUS 
                        if not pawn_count[LIGHT_0][FILE_[i]+1] then
                            score[DARK_1]+=ROOK_OPEN_FILE_BONUS 
                        end if
                    end if
                    if RANK_[i] = 7 then
                        score[DARK_1]+=ROOK_ON_SEVENTH_BONUS 
                    end if
                elsif piece[i] = KING then
                    if piece_mat[LIGHT_0]<=1200 then
                        score[DARK_1]+=king_endgame_pcsq[flip[i]] 
                    else
                        score[DARK_1]+=eval_dark_king(i) 
                    end if
                end if
            end if
        end if
    end for

    --/* the score[] array is set, now return the score relative
    --   to the side to move */

    if side = LIGHT then
        return score[LIGHT_0]-score[DARK_1]
    end if

    return score[DARK_1]-score[LIGHT_0]

end function       

