-- screen.e

include button.e
include data.e
include mouse.e

-----------------------------------------------------------------------------
sequence vc                  
vc = {}

procedure video(integer v)                                         
-- video_config():
--  Return a sequence of values describing the current video
--  configuration:
--  {color monitor?, graphics mode, text rows, text columns,
--  xpixels, ypixels, number of colors, number of pages}
  
--  The following constants are defined in graphics.e:
  
--  global constant VC_COLOR = 1,
--  VC_MODE  = 2,
--  VC_LINES = 3,
--  VC_COLUMNS = 4,
--  VC_XPIXELS = 5,
--  VC_YPIXELS = 6,
--  VC_NCOLORS = 7,
--  VC_PAGES = 8
    if not graphics_mode(v) then
        vc = video_config()
    else
        puts(1,"This video mode is lacking\n")
    end if
end procedure

procedure end_video()
    if graphics_mode(-1) then
    end if
end procedure

procedure square(integer x, integer y, integer len, 
                 integer color, integer fill)
    polygon(color,fill,{{x,y},{x+len,y},{x+len,y+len},{x,y+len}})    
end procedure                                      

function load_image(sequence s)
--  BMP_OPEN_FAILED = 1,
--  BMP_UNEXPECTED_EOF = 2,
--  BMP_UNSUPPORTED_FORMAT = 3
    object x
    x = read_bitmap(s)
    if atom(x) then
        return x
    end if         
    return x[2]
end function

procedure show_image(integer x, integer y, object img)
    if atom(img) then
    --  BMP_OPEN_FAILED = 1,
    --  BMP_UNEXPECTED_EOF = 2,
    --  BMP_UNSUPPORTED_FORMAT = 3
        if img = 1 then
            puts(1,"BMP_OPEN_FAILED\n")
        elsif img = 2 then
            puts(1,"BMP_UNEXPECTED_EOF\n")
        elsif img = 3 then
            puts(1,"BMP_UNSUPPORTED_FORMAT\n")
        end if
    else
        display_image({x,y},img)
    end if
end procedure                                                               

----------------------------------------------------------------------------

global constant WCOLOR = BROWN -- color  of white pieces
global constant BSQCOLOR = GREEN  -- color of black squares
global constant WSQCOLOR = BRIGHT_WHITE -- color of white squares
global constant ANNOTCOLOR = BRIGHT_WHITE -- color of annotations

global integer back,capture_piece,ep_square
back = 0
capture_piece = 0
ep_square = 0

global integer last_move_side
last_move_side = 0

global integer rotate
rotate = 0

constant XO=50,YO=50,DM=32


constant CO = {
 1, 2, 1, 2, 1, 2, 1, 2,
 2, 1, 2, 1, 2, 1, 2, 1,
 1, 2, 1, 2, 1, 2, 1, 2,
 2, 1, 2, 1, 2, 1, 2, 1,
 1, 2, 1, 2, 1, 2, 1, 2,
 2, 1, 2, 1, 2, 1, 2, 1,
 1, 2, 1, 2, 1, 2, 1, 2,
 2, 1, 2, 1, 2, 1, 2, 1
}

object bpwn,bknt,bbsp,brok,bqun,bkng
object wsqr,bsqr

sequence BO_COORD
sequence PI_LIST

procedure load_pieces()
    bpwn = load_image("pawn.bmp")
    bknt = load_image("knight.bmp")
    bbsp = load_image("bishop.bmp")
    brok = load_image("rook.bmp")
    bqun = load_image("queen.bmp")
    bkng = load_image("king.bmp")
end procedure

procedure init_screen_application()
    integer k
    load_pieces()
    PI_LIST = {bpwn,bknt,bbsp,brok,bqun,bkng}
    BO_COORD = repeat({0,0},64)
    k = 1
    for i=0 to 7 do
        for j=0 to 7 do
            BO_COORD[k] = {XO+j*DM,YO+i*DM}
            k += 1
        end for
    end for
end procedure

global
procedure rotate_coord()
    integer k,z
    if rotate then
        k = 64
        z = -1
    else
        k = 1
        z = 1
    end if
    for i=0 to 7 do
        for j=0 to 7 do
            BO_COORD[k] = {XO+j*DM,YO+i*DM}
            k += z
        end for
    end for
end procedure

integer done_once
done_once = 0

procedure draw_clean_board()    
    integer k
    sequence c,L,N
    polygon(CYAN,1,{{20,50},{350,50},{350,350},{20,350}})
    if rotate then
        L = {"H","G","F","E","D","C","B","A"}
        N = {"1","2","3","4","5","6","7","8"}
    else
        L = {"A","B","C","D","E","F","G","H"}
        N = {"8","7","6","5","4","3","2","1"}
    end if
    c = {WSQCOLOR,BSQCOLOR}
    k = 1
    set_foreground(ANNOTCOLOR)
    for i=0 to 7 do
        for j=0 to 7 do
            setx(30) sety(YO+i*DM+10)
            write(N[i+1])
            square(XO+j*DM,YO+i*DM,DM,c[remainder(i+j,2)+1],1)
        end for
    end for
    if not done_once then
        wsqr = save_image({XO,YO},{XO+DM-1,YO+DM-1})
        bsqr = save_image({XO+DM,YO},{XO+2*DM-1,YO+DM-1})
        done_once = 1
    end if
    set_foreground(ANNOTCOLOR)
    for i=1 to 8 do
        setx(32+i*32) sety(312)
        write(L[i])
    end for
end procedure         



function fool(sequence s, integer color)
    for i=1 to DM do
        for j= 1 to DM do
            if s[i][j] = 0 then
                s[i][j] = color
            end if
        end for
    end for
    return s
end function

global
procedure select_square(integer idx, integer color)
    integer p
    if rotate then
        idx = 65 - idx
    end if
    p = piece[idx]
    if p=EMPTY then
        return
    else
        p = p + 1 
    end if
    if idx < 1 or idx > 64 then
        return
    end if
    if side = 0 and CO[idx] = 1 then
        show_image(BO_COORD[idx][1],BO_COORD[idx][2],
                   fool(and_bits(PI_LIST[p],wsqr),color))
    elsif side = 0 and CO[idx] = 2 then
        show_image(BO_COORD[idx][1],BO_COORD[idx][2],
                   fool(and_bits(PI_LIST[p],bsqr),color))
    elsif side = 1 and CO[idx] = 1 then
        show_image(BO_COORD[idx][1],BO_COORD[idx][2],
                   fool(and_bits(PI_LIST[p],wsqr),color))
    elsif side = 1 and CO[idx] = 2 then
        show_image(BO_COORD[idx][1],BO_COORD[idx][2],
                   fool(and_bits(PI_LIST[p],bsqr),color))
    end if
end procedure                       


procedure erase(integer sq)
    integer color,x,y
    color = CO[sq]
    x = BO_COORD[sq][1]
    y = BO_COORD[sq][2]
    if color = 1 then
        show_image(BO_COORD[sq][1],BO_COORD[sq][2],wsqr)
    else    
        show_image(BO_COORD[sq][1],BO_COORD[sq][2],bsqr)
    end if
end procedure


procedure draw(integer p,integer sq)
    integer color,x,y
    color = CO[sq]
    x = BO_COORD[sq][1]
    y = BO_COORD[sq][2] 
    if color = 1 and last_move_side = 0 then
        show_image(BO_COORD[sq][1],BO_COORD[sq][2],
                   fool(and_bits(PI_LIST[p],wsqr),WCOLOR))
    elsif color = 2 and last_move_side = 0 then
        show_image(BO_COORD[sq][1],BO_COORD[sq][2],
                   fool(and_bits(PI_LIST[p],bsqr),WCOLOR))
    elsif color = 1 and last_move_side = 1 then
        show_image(BO_COORD[sq][1],BO_COORD[sq][2],and_bits(PI_LIST[p],wsqr))
    elsif color = 2 and last_move_side = 1 then
        show_image(BO_COORD[sq][1],BO_COORD[sq][2],and_bits(PI_LIST[p],bsqr))
    end if
end procedure


procedure screen_regular_move_back(integer p, integer f, integer t)
    erase(t)
    draw(p+1,f)                          
    if capture_piece != EMPTY then
        last_move_side = xor_bits(last_move_side,1)
        draw(capture_piece+1,t)
        last_move_side = xor_bits(last_move_side,1)
    end if               
    capture_piece = EMPTY
end procedure

procedure screen_regular_move(integer p, integer f, integer t)
    erase(f)
    draw(p+1,t)
end procedure

procedure screen_promotion_move(integer p, integer f, integer t)
    screen_regular_move(p,f,t)
end procedure

procedure screen_castling_move(integer special)
    integer p1,p2,f1,t1,f2,t2
    if special = 1000 then
        p1 = 5  p2 = 3
        f1 = 61 t1 = 63
        f2 = 64 t2 = 62 
    elsif special = 2000 then
        p1 = 5  p2 = 3
        f1 = 61 t1 = 59
        f2 = 57 t2 = 60 
    elsif special = 3000 then
        p1 = 5  p2 = 3
        f1 = 5 t1 = 7
        f2 = 8 t2 = 6 
    elsif special = 4000 then
        p1 = 5  p2 = 3
        f1 = 5 t1 = 3
        f2 = 1 t2 = 4 
    end if
    screen_regular_move(p1,f1,t1)
    screen_regular_move(p2,f2,t2)
end procedure

procedure screen_castling_move_back(integer special)
    integer p1,p2,f1,t1,f2,t2
    if special = 1000 then
        p1 = 5  p2 = 3
        f1 = 61 t1 = 63
        f2 = 64 t2 = 62 
    elsif special = 2000 then
        p1 = 5  p2 = 3
        f1 = 61 t1 = 59
        f2 = 57 t2 = 60 
    elsif special = 3000 then
        p1 = 5  p2 = 3
        f1 = 5 t1 = 7
        f2 = 8 t2 = 6 
    elsif special = 4000 then
        p1 = 5  p2 = 3
        f1 = 5 t1 = 3
        f2 = 1 t2 = 4 
    end if
    screen_regular_move_back(p1,f1,t1)
    screen_regular_move_back(p2,f2,t2)
end procedure

procedure screen_en_passant_move_back(integer f, integer t)
    erase(t)
    draw(1,f)
    last_move_side = xor_bits(last_move_side,1)
    if last_move_side = 0 then
        draw(1,t-8)
    else    
        draw(1,t+8)
    end if
    last_move_side = xor_bits(last_move_side,1)
end procedure

procedure screen_en_passant_move(integer f, integer t, integer eps)
    erase(f)
    draw(1,t)
    erase(eps)
end procedure

procedure screen_promotion(integer np,integer f, integer t)
    erase(f)
    draw(np+1,t)
end procedure

procedure screen_promotion_back(integer f, integer t)
    erase(t)
    draw(1,f)
end procedure


global
procedure screen_move(sequence m)
    integer f,t,p
    f = m[FROM] 
    t = m[TO]  
    if not back then
        p = piece[t]
        if p=EMPTY then
           return 
        end if
        if and_bits(m[BITS],32) then -- promotion
            screen_promotion(m[PROMOTE],f,t)
        elsif and_bits(m[BITS],4) then -- en passant
            if side!=LIGHT then
               screen_en_passant_move(f,t,t+8)
            elsif side!=DARK then
               screen_en_passant_move(f,t,t-8)
            end if
        elsif and_bits(m[BITS],2) then -- castle
            if m[TO] = 63 then
                screen_castling_move(1000)
            elsif m[TO] = 59 then
                screen_castling_move(2000)
            elsif m[TO] = 7 then
                screen_castling_move(3000)
            elsif m[TO] = 3 then
                screen_castling_move(4000)
            end if      
        else    
            screen_regular_move(p,f,t)
        end if
    else
        p = piece[f]
        if p=EMPTY then
           return 
        end if
        if and_bits(m[BITS],32) then -- promotion
            screen_promotion_back(f,t)
        elsif and_bits(m[BITS],4) then -- en passant
               screen_en_passant_move_back(f,t)
        elsif and_bits(m[BITS],2) then -- castle
            if m[TO] = 63 then
                screen_castling_move_back(1000)
            elsif m[TO] = 59 then
                screen_castling_move_back(2000)
            elsif m[TO] = 7 then
                screen_castling_move_back(3000)
            elsif m[TO] = 3 then
                screen_castling_move_back(4000)
            end if      
        else    
            screen_regular_move_back(p,f,t)
        end if
    end if
end procedure

global
procedure draw_screen_position()
    integer k,p,z
    draw_clean_board()
    if rotate then
        k = 64
        z = -1
    else      
        k = 1
        z = 1
    end if
    for i=0 to 7 do
        for j=0 to 7 do
            p = piece[k]+1
            if Color[k] = 0 and CO[k] = 1 then
                show_image(XO+j*DM,YO+i*DM,fool(and_bits(PI_LIST[p],wsqr),WCOLOR))
            elsif Color[k] = 0 and CO[k] = 2 then
                show_image(XO+j*DM,YO+i*DM,fool(and_bits(PI_LIST[p],bsqr),WCOLOR))
            elsif Color[k] = 1 and CO[k] = 1 then
                show_image(XO+j*DM,YO+i*DM,and_bits(PI_LIST[p],wsqr))
            elsif Color[k] = 1 and CO[k] = 2 then
                show_image(XO+j*DM,YO+i*DM,and_bits(PI_LIST[p],bsqr))
            end if
            k += z
        end for
    end for
end procedure


global
procedure draw_promo_pieces(integer side, integer x, integer y)
    integer k,p
    sequence bg
    sequence PROMOTION 
    PROMOTION = {2,3,4,5,-2,-3,-4,-5}
    polygon(CYAN,1,{{x,y},{x+4*DM,y},{x+4*DM,y+DM},{x,y+DM}})
    bg = save_image({x,y},{x+DM-1,y+DM-1})
    k = 1+(side=1)*4
    for i=0 to 3 do
        p = PROMOTION[k]
        if p > 0 then
            show_image(x+i*DM,y,fool(and_bits(PI_LIST[p],bg),WCOLOR))
        elsif p < 0 then
            show_image(x+i*DM,y,and_bits(PI_LIST[-p],bg))
        end if
        k += 1
    end for
end procedure


--constant BX1=400,BX2=500,BY1=150,BY2=400
constant BX1=500,BX2=600,BY1=60,BY2=310

constant b_list = {
{LABEL_BUTTON,"COMPUTER"},
{LABEL_BUTTON,"PLAY"},
{LABEL_BUTTON,"LEVEL"},
{LABEL_BUTTON,"BOOK"},
{LABEL_BUTTON,"NEW"},
{LABEL_BUTTON,"ROTATE"},
{LABEL_BUTTON,"TAKEBACK"},
{LABEL_BUTTON,"HINT"},
{LABEL_BUTTON,"QUIT"}
}

sequence mess_list

global
procedure start_screen_application()
    init_screen_application()
    video(18)
    polygon(CYAN,1,{{0,0},{vc[VC_XPIXELS],0},
                {vc[VC_XPIXELS],vc[VC_YPIXELS]},{0,vc[VC_YPIXELS]}})
    draw_screen_position()
    mess_list = createbuttons(BX1,BY1,b_list,8) 
end procedure

global
procedure terminate_screen_application()
    end_video()
    puts(1,"\nTranslated from C to Euphoria\n")
    puts(1,"and added GUI by Normand M. Blais\n")
end procedure

-----------------------------------------------------------------------------


global constant EVMOUSE = 1, EVKEYBOARD = 2, EVMOVE = 3

global constant NEW = 1001, QUIT = 1002, ESC = 1003, COMPUTER = 1004,
                BACK = 1005, ROTATE = 1006, HINT = 1007, BOOK = 1008,
                LV = 1009, PLAY = 1010
                

global constant EVLEFT  = 
{COMPUTER,PLAY,LV,BOOK,NEW,ROTATE,HINT,BACK,QUIT}

constant MSEVENT = 1
constant MS_X    = 2
constant MS_Y    = 3

global sequence eventmessage  -- mouse, keyboard, move
global sequence mess

mess = {}
eventmessage={0,0,0}

global integer bd_square


global
function geteventmessage()
    object event
    integer r,c
    while 1 do
        event = get_mouse()
        if sequence(event) and and_bits(event[MSEVENT],LEFT_DOWN) then
            if event[MS_X] > XO and event[MS_X] < XO+8*DM and
               event[MS_Y] > YO and event[MS_Y] < YO+8*DM then
                r = floor((event[MS_Y] - YO)/DM) + 1
                c = floor((event[MS_X] - XO)/DM) + 1 
                bd_square = 8*(r-1)+c
                event = {0,0,MOVE} 
                exit
            elsif event[MS_X] > BX1 and event[MS_X] < BX2 and
               event[MS_Y] > BY1 and event[MS_Y] < BY2 then
               mess = selectbutton(event[MS_X],event[MS_Y],mess_list)
               if equal(mess,"COMPUTER") then
                   event = {COMPUTER,0,0}
               elsif equal(mess,"PLAY") then
                   event = {PLAY,0,0}
               elsif equal(mess,"LEVEL") then
                   event = {LV,0,0}
--               elsif equal(mess,"BOOK") then
--                   event = {0,0,0}
               elsif equal(mess,"NEW") then
                   event = {NEW,0,0}
               elsif equal(mess,"ROTATE") then
                   event = {ROTATE,0,0}
               elsif equal(mess,"TAKEBACK") then
                   event = {BACK,0,0}
               elsif equal(mess,"HINT") then
                   event = {HINT,0,0}
               elsif equal(mess,"QUIT") then
                   event = {QUIT,0,0}
               end if
               return event
            end if
        end if
        event = get_key()
        if event = 27 then
            event = {0,ESC,0}
            exit
        end if
    end while      
    return event
end function            


global
function promote(integer side)
    object event
    integer x,y,idx
    x = XO+8*DM+10
    y = YO+25        
    mouse_pointer(0)
    draw_promo_pieces(side,x,y)
    mouse_pointer(1)
    while 1 do
        event = get_mouse()
        if sequence(event) and and_bits(event[MSEVENT],LEFT_DOWN) then
            if event[MS_X] >= x and event[MS_X] <= x+4*DM and
               event[MS_Y] >= y and event[MS_Y] <= y+DM then
                idx = floor((event[MS_X] - x)/DM) + 1
                if idx >= 1 and idx <= 4 then                
                    mouse_pointer(0)
                    polygon(CYAN,1,{{x-1,y-1},
                                {x+4*DM,y-1},{x+4*DM,y+DM+1},{x-1,y+DM+1}})
                    mouse_pointer(1)
                    return idx
                end if
            end if
        end if
    end while      
    return 0
end function
    
global
procedure logo()
    setx(30) sety(10)
    set_foreground(YELLOW) 
    write("Tom Kerrigan's Simple Chess Program (TSCP)")
end procedure

constant SB = {"COMPUTER","LEVEL","BOOK","TURN"}
sequence sb
sb = {0,1,0,0}

constant SBX = 325, SBY = 325

global
procedure status_bar()
    sety(10+SBY)
    set_foreground(MAGENTA) 
    setx(SBX+5) 
    write("   ")
    setx(SBX+30) 
    for i=1 to length(SB) do
        write(SB[i]&"   ")
    end for
end procedure

global
procedure update_status_bar(integer item, object x1 , object x2)
    sequence c,x
    x = {x1,x2}
    c = {CYAN,YELLOW}
    sety(25+SBY)         
    for i=1 to 2 do
    set_foreground(c[i]) 
    if item = 1 then -- COMPUTER
        setx(SBX+55)
        if x[i] = EMPTY then
            write("OFF")
        else
            write("ON")
        end if
    elsif item = 2 then -- LEVEL
        setx(SBX+115)
        write(sprintf("%3d",x[i]))
    elsif item = 3 then  -- BOOK
        setx(SBX+185)
        if x[i] then
            write("ON")
        else    
            write("OFF")
        end if
    elsif item = 4 then -- TURN
        setx(SBX+235)
        if x[i] then
            write("BLACK")
        else    
            write("WHITE")
        end if
    end if
    end for
end procedure

